/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.federation.servlet;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jboss.security.federation.service.Trust;
import org.jboss.security.federation.service.TrustService;

/**
 * The Federation Server's Trust Service.
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Oct 31, 2006
 *
 */
public class SSOTrustServer extends HttpServlet 
{
    /**
     * 
     */
    private TrustService trustService = null;
    
    /**
     * 
     */
    public void init() throws ServletException 
    {        
        super.init();
        trustService = Trust.getInstance();
    }

    /**
     * 
     */
    protected void doGet(HttpServletRequest request,HttpServletResponse response) throws ServletException, IOException 
    {
        this.doPost(request,response);
    }

    /**
     * 
     */
    protected void doPost(HttpServletRequest request,HttpServletResponse response) throws ServletException, IOException 
    { 
        String contextPath = request.getContextPath();
        String requestURI = request.getRequestURI();
        String actionPath = requestURI.substring(contextPath.length());
        
        if(actionPath.endsWith("generateSecret"))
        {
            String identifier = request.getParameter("identifier");
            String secret = this.trustService.generateSecret(identifier);
            response.getOutputStream().write(secret.getBytes());
            response.getOutputStream().flush();
        }
        else if(actionPath.endsWith("validateSecret"))
        {
            String identifier = request.getParameter("identifier");
            String secret = request.getParameter("secret");
            String validationStatus = this.trustService.validateSecret(identifier,secret);
            response.getOutputStream().write(validationStatus.getBytes());
            response.getOutputStream().flush();
        }               
    }    
}
