/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;

import java.io.Serializable;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.Collection;

import org.jboss.security.util.Tools;

/**
 * Identity represents the identity information related to the user being authenticated by the Single Sign On engine. This object is a key component
 * of the system and is used by other parts of the system where identity related information needs to be processed. It tends to cover most of the information
 * needed to associate with a user's profile. However, applications have the option to extend this class in order to integrate custom data
 * that needs to be identity information as per their application requirements
 * 
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class Identity implements Serializable
{
    public static final String NULL = "null";
    
	private String 		username = null;
	private byte[] 		password = null;
	private String 		email = null;
	private boolean 	active = false;	
    private String fullName = null;    
    private Collection otherEmails = null;
    private Collection phoneNumbers = null;
    
    private String companyName = null;
    private String companyRole = null;
    private String companyTitle = null;
    
    private Address companyAddress = null;
                
    
    private Collection roles = null; //a collection of strings representing roles that this identity belongs to
	
    /**
     * Initializes the Identity object
     *
     */
	public Identity()
	{
		super();
	}
	
	/**
     * 
     * @return
	 */
	public String getUserName()
	{
		return this.username;
	}
	
    /**
     * 
     * @param username
     */
	public void setUserName(String username)
	{
		this.username = username;
	}
	
    /**
     * 
     * @return
     */
	public byte[] getPassword()
	{
		return this.password;
	}
	
    /**
     * 
     * @param password
     */
	public void setPassword(byte[] password)
	{
		this.password = password;
	}
	
    /**
     * 
     * @return
     */
	public boolean isActive()
	{
		return this.active;
	}
	
    /**
     * 
     * @param active
     */
	public void setActive(boolean active)
	{
		this.active = active;
	}
	
    /**
     * 
     * @return
     */
	public String getEmail()
	{
		return this.email;
	}
	
    /**
     * 
     * @param email
     */
	public void setEmail(String email)
	{
		this.email = email;
	}
    
    /**
     * 
     * @return
     */
    public String getFullName()
    {
        return this.fullName;
    }
    
    /**
     * 
     * @param fullName
     */
    public void setFullName(String fullName)
    {
        if(fullName!=null && fullName.trim().equalsIgnoreCase(NULL))
        {
            fullName = null;
        }
        this.fullName = fullName;
    }
       
    /**
     * 
     * @return
     */
    public Collection getOtherEmails()
    {
        if(this.otherEmails==null)
        {
            this.otherEmails = new ArrayList();
        }
        return this.otherEmails;
    }
    
    /**
     * 
     * @param otherEmails
     */
    public void setOtherEmails(Collection otherEmails)
    {
        this.otherEmails = otherEmails;
    }
    
    /**
     * 
     * @return
     */
    public Collection getPhoneNumbers()
    {
        if(this.phoneNumbers==null)
        {
            this.phoneNumbers = new ArrayList();
        }
        return this.phoneNumbers;
    }
    
    /**
     * 
     * @param phoneNumbers
     */
    public void setPhoneNumbers(Collection phoneNumbers)
    {
        this.phoneNumbers = phoneNumbers;
    }
    
    /**
     * 
     * @param phoneNumber
     */
    public void addPhoneNumber(String phoneNumber)
    {
        if(!Tools.isEmpty(phoneNumber))
        {
            this.getPhoneNumbers().add(phoneNumber);
        }
    }
    
    /**
     * 
     * @param email
     */
    public void addEmail(String email)
    {
        if(!Tools.isEmpty(email))
        {
            this.getOtherEmails().add(email);
        }
    }
    
    /**
     * 
     * @return
     */
    public String getCompanyName()
    {
        return this.companyName;
    }
    
    /**
     * 
     * @param companyName
     */
    public void setCompanyName(String companyName)
    {
        if(companyName!=null && companyName.trim().equalsIgnoreCase(NULL))
        {
            companyName = null;
        }
        this.companyName = companyName;
    }
    
    /**
     * 
     * @return
     */
    public String getCompanyRole()
    {
        return this.companyRole;
    }
    
    /**
     * 
     * @param companyRole
     */
    public void setCompanyRole(String companyRole)
    {
        if(companyRole!=null && companyRole.trim().equalsIgnoreCase(NULL))
        {
            companyRole = null;
        }
        this.companyRole = companyRole;
    }
    
    /**
     * 
     * @return
     */
    public String getCompanyTitle()
    {
        return this.companyTitle;
    }
    
    /**
     * 
     * @param companyTitle
     */
    public void setCompanyTitle(String companyTitle)
    {
        if(companyTitle!=null && companyTitle.trim().equalsIgnoreCase(NULL))
        {
            companyTitle = null;
        }
        this.companyTitle = companyTitle;
    }
    
    /**
     * 
     * @return
     */
    public Address getCompanyAddress()
    {
        if(this.companyAddress==null)
        {
            this.companyAddress = new Address();
        }
        return this.companyAddress;
    }
    
    /**
     * 
     * @param companyAddress
     */
    public void setCompanyAddress(Address companyAddress)
    {
        if(companyAddress.getStreet()!=null && companyAddress.getStreet().trim().equalsIgnoreCase(NULL))
        {
            companyAddress.setStreet(null);
        }
        if(companyAddress.getCity()!=null && companyAddress.getCity().trim().equalsIgnoreCase(NULL))
        {
            companyAddress.setCity(null);
        }
        if(companyAddress.getState()!=null && companyAddress.getState().trim().equalsIgnoreCase(NULL))
        {
            companyAddress.setState(null);
        }       
        if(companyAddress.getPostalCode()!=null && companyAddress.getPostalCode().trim().equalsIgnoreCase(NULL))
        {
            companyAddress.setPostalCode(null);
        }
        if(companyAddress.getCountry()!=null && companyAddress.getCountry().trim().equalsIgnoreCase(NULL))
        {
            companyAddress.setCountry(null);
        }
        this.companyAddress = companyAddress;
    }   
    
    /**
     * Returns the set of roles that this identity belongs to in the system
     * 
     * @param provider the LoginProvider that provides access to the Identity store
     * @return roles
     */
    public Collection getRoles(LoginProvider provider)
    {
       if(this.roles==null)
       {
           //load the roles for this user from the identity store
           Collection allRoles = provider.readAllRoles();
           this.roles = new ArrayList();
           if(!Tools.isEmpty(allRoles))
           {
               for(Iterator itr=allRoles.iterator();itr.hasNext();)
               {
                   Role role = (Role)itr.next();
                   this.addRole(role,this.roles); 
               }
           }
       }
       return this.roles;
    }
    
    /**
     * 
     * @param role 
     * @param roles 
     */
    private void addRole(Role role,Collection roles)
    {
        Collection members = role.getIdentities();
        
        if(!Tools.isEmpty(members))
        {
            for(Iterator itr=members.iterator();itr.hasNext();)
            {
                String member = (String)itr.next();
                if(member.equals(this.username))
                {
                    roles.add(role.getName());
                    break;
                }
            }
        }
    }
}
