/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.idm;

import java.io.Serializable;
import java.util.Collection;
import java.util.ArrayList;

/**
 * Role represents an authorization group used by an Authorization system to grant appropriate access.
 * A role consists of users that are considered members of that role.
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - Jun 1, 2006
 *
 */
public class Role implements Serializable
{
    /**
     * 
     */
    private String name = null;
    private Collection identities = null; //a collection of strings representing the identities associated with this role
    
    /**
     * Initializes a role object
     *
     */
    public Role()
    {        
    }

    /**
     * Return the role name
     * 
     * @return
     */
    public String getName() 
    {
        return name;
    }

    /**
     * Sets the name of the role
     * 
     * @param name
     */
    public void setName(String name) 
    {
        this.name = name;
    }
    
    /**
     * Return a collection of identities which represent all users/members that belong to this role
     * 
     * @return
     */
    public Collection getIdentities()
    {
       if(this.identities==null)
       {
           this.identities = new ArrayList();
       }
       return this.identities;
    }
    
    /**
     * Sets a collection of identities which represent all users/members that belong to this role
     * 
     * @param identities
     */
    public void setIdentities(Collection identities)
    {
        this.identities = identities;
    }
}
