/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.saml;

import java.io.Serializable;

/**
 * AuthResponse represents data related to an authentication assertion performed by one of the members of the Single Sign On Federation.
 * 
 * 
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class AuthResponse implements Serializable
{
    private String assertingParty = null;
    private String assertToken = null;
    private SSOUser user = null;    
    private boolean success = false;    
    
    /**
     * Initializes a response
     * 
     * @param assertingParty a member/partner web application of the Single Sign On Federation
     * @param assertToken raw String representation of the token that was issued by the assertingParty
     * @param user the user being considered for authentication
     * @param success status of the assertion
     */
    public AuthResponse(String assertingParty,String assertToken,SSOUser user,boolean success)
    {
        super();
        this.assertingParty = assertingParty;
        this.assertToken = assertToken;
        this.user = user;
        this.success = success;        
    }
    
    /**
     * Returns the asserting party which is a member/partner web application in the Single Sign On Federation
     * 
     * @return
     */
    public String getAssertingParty()
    {
        return this.assertingParty;
    }
    
    /**
     * Sets the asserting party which is a member/partner web application in the Single Sign On Federation
     * 
     * @param assertingParty
     */
    public void setAssertingParty(String assertingParty)
    {
        this.assertingParty = assertingParty;
    }
    
    /**
     * Returns the assertion token
     * 
     * @return
     */
    public String getAssertToken()
    {
        return this.assertToken;
    }
    
    /**
     * Return the user being considered for authentication
     * 
     * @return
     */
    public SSOUser getUser()
    {
        return this.user;
    }
    
    /**
     * Sets the user being considered for authentication
     * 
     * @param user
     */
    public void setUser(SSOUser user)
    {
        this.user = user;
    }
    
    /**
     * Returns the status of whether the asserting party successfully authenticated this user or not
     * 
     * @return
     */
    public boolean isAuthenticated()
    {
        return this.success;
    }    
}
