/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package test.jboss.security.saml;


import org.apache.cactus.*;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import org.jboss.security.saml.*;


/**
 * 
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class MarshalPartnersTestCase extends ServletTestCase
{
    /**
     * 
     * @throws Exception
     */
    public void test() throws Exception
    {       
       SingleSignOn sso = SSOManager.findProcessor();
       
       //load the partners...this should be loaded
		Map partners = new HashMap();
		partners.put(".jboss.com", "http://node1.jboss.com:8080/federate");
		partners.put(".jboss.org", "http://node1.jboss.org:8080/federate");
      
              
       String marshalled = sso.marshalPartners(partners);
       System.out.println("Marshalled Partners ="+marshalled);
       
       //assert the xml produced from the map of partners
       this.assertNotNull(marshalled); 
       boolean contains1 = marshalled.indexOf(".jboss.com")!=-1;
       boolean contains2 = marshalled.indexOf("http://node1.jboss.com:8080/federate")!=-1;
       boolean contains3 = marshalled.indexOf("http://node1.jboss.org:8080/federate")!=-1;
       boolean contains4 = marshalled.indexOf(".jboss.org")!=-1;
       this.assertTrue(contains1);
       this.assertTrue(contains2);
       this.assertTrue(contains3);
       this.assertTrue(contains4);
       
       
       //now unmarsal this xml back into a Map of partners and see
       //of the original map is restored
       Map unmarshalled = sso.unmarshalPartners(marshalled);
       
       //assert the unmarshalled info
       this.assertNotNull(unmarshalled);
       this.assertTrue(unmarshalled.containsKey(".jboss.com"));
       this.assertTrue(unmarshalled.containsKey(".jboss.org"));
       this.assertTrue(unmarshalled.containsValue("http://node1.jboss.com:8080/federate"));
       this.assertTrue(unmarshalled.containsValue("http://node1.jboss.org:8080/federate"));
       
       System.out.println("---------------------------------------------");
       for(Iterator itr=unmarshalled.keySet().iterator();itr.hasNext();)
       {
    	   String domain = (String)itr.next();
    	   String federationServer = (String)unmarshalled.get(domain);
    	   System.out.println("Domain="+domain+",Server="+federationServer);
       }
       System.out.println("---------------------------------------------");
    }
}
