/******************************************************************************
 * JBoss, a division of Red Hat                                               *
 * Copyright 2006, Red Hat Middleware, LLC, and individual                    *
 * contributors as indicated by the @authors tag. See the                     *
 * copyright.txt in the distribution for a full listing of                    *
 * individual contributors.                                                   *
 *                                                                            *
 * This is free software; you can redistribute it and/or modify it            *
 * under the terms of the GNU Lesser General Public License as                *
 * published by the Free Software Foundation; either version 2.1 of           *
 * the License, or (at your option) any later version.                        *
 *                                                                            *
 * This software is distributed in the hope that it will be useful,           *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU           *
 * Lesser General Public License for more details.                            *
 *                                                                            *
 * You should have received a copy of the GNU Lesser General Public           *
 * License along with this software; if not, write to the Free                *
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA         *
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.                   *
 ******************************************************************************/
package org.jboss.security.valve;

import java.io.IOException;
import java.io.InputStream;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.io.IOException;
import java.net.URL;
import java.net.URLEncoder;

import javax.servlet.ServletException;

import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.valves.ValveBase;

import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.log4j.Logger;

import org.jboss.security.sso.util.SSOUtil;
import org.jboss.security.util.Tools;

/**
 * Created on : Feb 5, 2007
 * @author Sohil Shah - sohil.shah@jboss.com
 *
 */
public class SSOFederationRouter extends ValveBase
{
	private Map partners = null;
	private static final Logger log = Logger.getLogger(SSOFederationRouter.class);
		
	/**
	 * 
	 */
	public void invoke(Request request, Response response) throws IOException,
			ServletException
	{
		if(this.partners == null)
		{
			//load the partners...this should be loaded
			//by querying your domain's federation server
			try
			{
				this.partners = this.lookupPartners(request);
			}
			catch(Exception e)
			{
				log.error(this, e);
				this.partners = new HashMap();
			}
		}
		
		String referer = request.getHeader("Referer");
		
		
		//calculate if the referer is a sso federation member
		if(referer!=null && referer.trim().length()>0)
		{
			String server = request.getServerName();
			String routerParam = request.getParameter("router");
			String myDomain = null;
			String refererDomain = null;
			boolean routing = false;
			
			URL url = new URL(referer);
	        String host = url.getHost();
	        refererDomain = this.getDomain(host);
	        myDomain = this.getDomain(server);        
	        if(	
	        		refererDomain != null 
	        		
	        					&&
	        					
	        		!refererDomain.equals(myDomain)
	        		
	        					&&
	        					
	        		//and if refererDomain belongs to the trusted
	        		//SSO Federation that this application belongs to
	        		this.partners.get(refererDomain.substring(1)) != null
	        )
	        {
	        	routing = true;
	        }
	        
	        //decide if this incoming request needs to be routed through
			//the proper federation server or not
			if(routing && (routerParam==null || routerParam.trim().length()==0))
			{
				String fullRequest = request.getRequestURL().toString();
				
				//route through the federation server				
				String federationServerLink = ((String)this.partners.get(refererDomain.substring(1))) +
				"/route?router=true&target=" +
				URLEncoder.
				encode(fullRequest, "UTF-8");
				
				//route request through the proper federation server
				response.getOutputStream().write(this.getRouterResponse(federationServerLink, request)
				.getBytes());
	            response.getOutputStream().flush();
	            
				return;
			}
		}
		
		
		
		//forward in the valve chain
		this.getNext().invoke(request, response);
	}
	
	/**
	 * Looks up web applications that are registered as partners in the SSO Federation
	 * 
	 * @return
	 */
	private Map lookupPartners(Request request) throws Exception
	{
		Map partners = new HashMap();
		
		String url = "http://"+request.getServerName()+":"+request.getServerPort()+"/federate/partners";
		HttpClient client = new HttpClient();
		GetMethod get = new GetMethod(url);
		InputStream is = null;
		try
		{
			int statusCode = client.executeMethod(get);
            is = get.getResponseBodyAsStream();
            byte[] response = Tools.getBytes(is);
            
            String partnerXml = new String(response);
            partners = SSOUtil.unmarshalPartners(partnerXml);            
		}
		finally
		{
			if(is != null)
			{
				try{is.close();}catch(IOException ioe){log.error(this, ioe);}
			}
		}
		
		return partners;
	}
	
	/**
     * 
     * @param serverName
     * @return
     */
    private String getDomain(String serverName)
    {
        String domain = null;

        boolean looksLikeIP = serverName.matches("^([0-9]{1,3}\\.){3}[0-9]{1,3}$");

        if (looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            looksLikeIP = parts.length == 4;
            for (int i = 0; i < parts.length && looksLikeIP; i++)
            {
                try
                {
                    int xxx = Integer.parseInt(parts[i]);
                    if ((xxx < 0) || (xxx > 255))
                    {
                        looksLikeIP = false;
                    }
                }
                catch (NumberFormatException e)
                {
                    looksLikeIP = false;
                }
            }
        }

        if (!looksLikeIP)
        {
            String[] parts = serverName.split("\\.");
            if (parts.length > 2)
            {
                StringBuffer domainStringBuffer = new StringBuffer(".");
                domainStringBuffer.append(parts[parts.length-2]);
                domainStringBuffer.append(".");
                domainStringBuffer.append(parts[parts.length-1]);
                domain = domainStringBuffer.toString();
            }
        }

        return domain;
    }
    
    /**
     * 
     * @param signoutURL
     * @return
     */
    private String getRouterResponse(String routingURL,Request request)
    {
        String routerResponse = "";
        
        StringBuffer buffer = new StringBuffer();
        buffer.append("<html>"+"\n");
        buffer.append("<head>"+"\n");
        buffer.append("</head>"+"\n");
        buffer.append("<body onload=\"setTimeout('document.form1.submit()',1000);\">"+"\n");
        buffer.append("<form name=\"form1\" action=\""+routingURL+"\" method=\"post\">"+"\n");
        Enumeration parameterNames = request.getParameterNames();
        while(parameterNames.hasMoreElements())
        {
        	String name = (String)parameterNames.nextElement();
        	String value = request.getParameter(name);
        	
        	//exclude some sso control control parameters from the post params
        	if(name.equals("token") || name.equals("target"))
        	{
        		continue;
        	}
        	
        	buffer.append("<input type=\"hidden\" name=\""+name+"\" value=\""+value+"\"/>"+"\n");
        }
        buffer.append("</form>"+"\n");
        buffer.append("</body>"+"\n");
        buffer.append("</html>"+"\n");
        
        
        routerResponse = buffer.toString();
        
        return routerResponse;
    }
}
